-- Particles
--  _____ _____ _____
-- |     |     |     |
-- |  L  |  D  |  R  |
-- |  O  |  I  |  E  |
-- |  A  |  S  |  M  |
-- |  D  |  P  |  O  |
-- |     |  L  |  V  |
-- |  I  |  A  |  A  |
-- |  N  |  Y  |  L  |
-- |_____|_____|_____|

property("Background", COLOUR, 255,255,255) -- Background (default to white)
property("Particle", GRADIENT, 0.0, 0, 0, 255, 1.0, 255, 255, 255)-- Particle gradient (default blue -> white)
property("Size",INTEGER,5,1) -- Size of particle (default 5)
property("Seed", INTEGER, 500, 0) -- Seed for random number generator (default to 500)
property("Points",INTEGER,5,1) -- Number of particles (default 5)

-- Initialise variables
points={}
frame_count = 1
last_frame = 0
Rate = math.ceil(((3*width))/frames)

-- Initialise random number generator
math.randomseed(Seed)

-- New Point function, generates random xy coordinates
function new_point()
    points[#points+1] = {math.random(0-width,0-Size),math.random(0-Size,height+Size)}
end

-- Generate random X and Y positions
for i=1,Points do
    points[#points+1] = {math.random(0-width,2*width),math.random(0-Size,height+Size)}
end

-- Pixel Function, run for each pixel in effect
function pixel(frame, x, y)
valid_points={}
    if frame ~= last_frame then -- Do this if it is a new frame
        if frame_count == Rate then
            for i=1,#points do
                points[i][1] = points[i][1]+math.ceil(((3*width)+(2*Size))/frames)
                if points[i][1] > 2*width then -- If the point reaches the end of the removal area, reset the x coordinate to the load in area
                    points[i][1] = math.random(0-width,0-Size)
                end
            end
            frame_count = 1
        else
            frame_count = frame_count + 1
        end
        last_frame = frame
    end
    -- Check whether the current pixel is within any of the particles
    for i=1,#points do
        if dist_from_point(x,y,i)<1+Size then
            table.insert(valid_points,i)
        end
    end
    -- Check how many particles this pixel is within
    if #valid_points == 1 then -- If only 1 then colour is picked from the gradient
        p = valid_points[1]
        if dist_from_point(x,y,p) < 1 + Size then
            fraction = (1+Size-(dist_from_point(x,y,p)-1))/(1+Size+1)
            return Particle:lookup(1-fraction)
        end
    elseif #valid_points == 0 then -- If none then background is output
        return Background
    else -- Otherwise the pixel is within multiple pixels and the pixel will be the colour relevant to the closest particle
        dists={}
        for i=1,#valid_points do
            dists[i] = (1+Size-(dist_from_point(x,y,valid_points[i])-1))/(1+Size+1)
        end
        table.sort(dists)
        return Particle:lookup(1-dists[#dists])
    end
end

-- Function to determine the distance of a pixel from the centre of the particle
function dist_from_point(x1,y1,p)
    x2 = points[p][1]
    y2 = points[p][2]
    return dist(x1,y1,x2,y2)
end

--[[
Copyright 2016 Pharos Architectural Controls Limited. All rights reserved.

This sample content is made freely available for illustrative purposes only. Pharos Architectural Controls grants a non-exclusive copyright licence to use this sample content from which you can generate content tailored to your own specific needs for use with products from Pharos Architectural Controls. This sample content has not been thoroughly tested under all conditions and Pharos Architectural Controls cannot guarantee reliability or serviceability.

This sample content is provided "as is" without warranties of any kind. The implied warranties of non-infringement, merchantability and fitness for a particular purpose are expressly disclaimed. In no event shall Pharos Architectural Controls be liable to any party for direct, indirect, incidental or consequential damages arising from the use of this sample content.

For further information please contact support@pharoscontrols.com.
--]]